local VERSION = tonumber(("$Revision: 60658 $"):match("%d+"))

Spyglass = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceConsole-2.0",
                                          "AceEvent-2.0", "AceHook-2.1",
                                          "FuBarPlugin-2.0")
local Spyglass, self = Spyglass, Spyglass

Spyglass.version = "r" .. VERSION
Spyglass.revision = VERSION
Spyglass.date = ("$Date: 2008-02-09 03:50:46 -0500 (Sat, 09 Feb 2008) $"):match("%d%d%d%d%-%d%d%-%d%d")

local _G = _G

local L = AceLibrary("AceLocale-2.2"):new("Spyglass")
local IBL = AceLibrary('ItemBonusLib-1.0')

local UnitName = _G.UnitName
local UnitRace = _G.UnitRace
local UnitClass = _G.UnitClass
local UnitLevel = _G.UnitLevel
local GetGuildInfo = _G.GetGuildInfo
local CanInspect = _G.CanInspect

local SetPortraitToTexture = _G.SetPortraitToTexture
local SetPortraitTexture = _G.SetPortraitTexture

local PLAYER_LEVEL = _G.PLAYER_LEVEL
local GUILD_TITLE_TEMPLATE = _G.GUILD_TITLE_TEMPLATE

-- Localizations

L:RegisterTranslations("enUS", function() return {
	["Addon for looking deeper into an inspected person's gear"] = true,
	['Show Inspection Window'] = true,
	['Show the inspection window. (displaying the last person inspected)'] = true,
	['Store Data'] = true,
	['Keep user data.'] = true,
	['Clear Data'] = true,
	['Clear all the currently stored data'] = true,
	['Removing player info: %s'] = true,
	['Clearing all user data'] = true,
	['Enabled'] = true,
	['Disabled'] = true,
	['Toggle Spyglass'] = true,
	['Bonus Summary'] = true,
	['Fonts\\FRIZQT__.TTF'] = true,
	['Menu'] = true,
	['Menu Options'] = true,
	['Last Inspected Unit'] = true,
	['nobody'] = true,
	['Saved Units'] = true,
	["|cffeda55fClick|r to toggle the inspect window."] = true,
	["|cffeda55fDisabled|r"] = true,
	['Attributes'] = true,
	['Resistance'] = true,
	['Skills'] = true,
	['Combat'] = true,
	['Defenses'] = true,
	['Spells'] = true,
	['Health and Mana'] = true,
	["%d hp5"] = true,
	["%d mp5"] = true,
} end)

L:RegisterTranslations("deDE", function() return {
	["Addon for looking deeper into an inspected person's gear"] = "Addon um die Ausrüstung eines Spielers genauer zu betrachten.",
	['Show Inspection Window'] = 'Inspektionsfenster anzeigen',
	['Show the inspection window. (displaying the last person inspected)'] = 'Zeigt das Inspektionsfenster. (Zeigt den zuletzt betrachteten Spieler an)',
	['Store Data'] = 'Daten speichern',
	['Keep user data.'] = 'Speichert alle Benutzerdaten.',
	['Clear Data'] = 'Daten löschen',
	['Clear all the currently stored data'] = 'Löscht alle derzeit gespeicherten Daten',
	['Removing player info: %s'] = 'Removing player info: %s',  --is this in use?
	['Clearing all user data'] = 'Clearing all user data',  --is this in use?
	['Enabled'] = 'Aktiviert',
	['Disabled'] = 'Gesperrt',
	['Toggle Spyglass'] = 'Spyglassfenster öffnen/schließen',
	['Bonus Summary'] = 'Bonuszusammenfassung',
	['Fonts\\FRIZQT__.TTF'] = 'Fonts\\FRIZQT__.TTF',
	['Menu'] = 'Menü',
	['Menu Options'] = 'Menü Optionen',
	['Last Inspected Unit'] = 'Zuletzt betrachteter Spieler',
	['nobody'] = 'niemand',
	['Saved Units'] = 'gespeicherte Spieler',
	["|cffeda55fClick|r to toggle the inspect window."] = "|cffeda55fKlicken|r um das Inspektionsfenster zu öffen/schließen.",
	["|cffeda55fDisabled|r"] = "|cffeda55fGesperrt|r",
	['Attributes'] = 'Grundwerte',
	['Resistance'] = 'Widerstände',
	['Skills'] = 'Fertigkeiten',
	['Combat'] = 'Nah- und Distanzkampf',
	['Defenses'] = 'Verteidigung',
	['Spells'] = 'Zauber',
	['Health and Mana'] = 'Gesundheit & Mana',
	["%d hp5"] = "%d hp5",
	["%d mp5"] = "%d mp5",
} end)

L:RegisterTranslations("koKR", function() return {
	["Addon for looking deeper into an inspected person's gear"] = "살펴보기 기능에 대한 추가 정보 재공 애드온",
	['Show Inspection Window'] = '살펴보기 창 표시',
	['Show the inspection window. (displaying the last person inspected)'] = '살펴보기 창을 표시합니다. (지난 살펴보기 대상을 표시합니다)',
	['Store Data'] = '저장',
	['Keep user data.'] = '살펴본 대상의 정보를 저장합니다.',
	['Clear Data'] = '자료 초기화',
	['Clear all the currently stored data'] = '현재 저장된 모든 자료를 삭제합니다',
	['Removing player info: %s'] = '플레이어 정보 삭제: %s',
	['Clearing all user data'] = '모든 플레이어의 자료를 삭제합니다',
	['Enabled'] = '사용',
	['Disabled'] = '사용 안함',
	['Toggle Spyglass'] = 'Spyglass창 토글',
	['Bonus Summary'] = '추가 능력치 정보',
	['Fonts\\FRIZQT__.TTF'] = 'Fonts\\2002.TTF',
	['Menu'] = '설정',
	['Menu Options'] = '설정을 변경합니다',
	['Last Inspected Unit'] = '지난 살펴보기 대상',
	['nobody'] = '없음',
	['Saved Units'] = '저장된 대상',
	["|cffeda55fClick|r to toggle the inspect window."] = "|cffeda55f클릭|r시 살펴보기 창을 토글합니다.",
	["|cffeda55fDisabled|r"] = "|cffeda55f사용 안함|r",
	['Attributes'] = '능력치',
	['Resistance'] = '저항',
	['Skills'] = '숙련도',
	['Combat'] = '전투척도',
	['Defenses'] = '방어도',
	['Spells'] = '주문',
	['Health and Mana'] = '생명력과 마나',
	["%d hp5"] = '5초당 %d의 생명력',
	["%d mp5"] = '5초당 %d의 마나',
} end)


function Spyglass:OnInitialize()
    self:RegisterDB('SpyglassDB')
    self:RegisterDefaults('profile', {
        showBonuses = true,
        lastInspected = nil,
        storeEquipment = false,
        storedEquipment = {}
    })
	
	self.unit = 'player'
	self.unitName = UnitName('player')
	self.currentInfo = nil
	self.inInspectRange = true
	
	self.latched = false
	
	self.inspectFrame = nil
	
	-- FuBar Config
	self:InitializeMenu()
	
	-- Set the binding header name
	_G['BINDING_HEADER_Spyglass'] = 'Spyglass'
	
	-- Set the binding name
	_G['BINDING_NAME_Toggle Spyglass'] = L['Toggle Spyglass']
	
	if self.db.profile.storeEquipment then
	    self.unitCache = self.db.profile.storedEquipment
	else
	    self.unitCache = {}
	end
end

function Spyglass:ADDON_LOADED(addonName)
    if addonName == 'Blizzard_InspectUI' then
        self.latched = self:LatchOn()
    end
end

function Spyglass:OnEnable(first)
    if not _G.InspectFrame then
        self:RegisterEvent('ADDON_LOADED')
    else
        self.latched = self:LatchOn()
    end
    
    -- Update FuBar
    self:Update()
end

function Spyglass:OnDisable()
    self.latched = not self:LetGo()
end

function Spyglass:GlueMovementFrame()
    local moveFrame = self.movement
    local playerModel = _G.InspectModelFrame
    
    if not moveFrame then
        moveFrame = CreateFrame("Frame", "InspectDragger", playerModel)
        moveFrame:SetFrameLevel(self.inspectFrame:GetFrameLevel() + 1)
        
        moveFrame.playerModel = playerModel
        
        -- Scales in OnMouseUp multiplied by scales in OnUpdate == 1.
        -- This is to ensure 1/1 pixel/movement
        
        moveFrame:SetScript("OnMouseDown", function()
            if arg1 == 'LeftButton' then
                this:StartMoving()
                this.isRotating = true
            elseif arg1 == 'RightButton' then
                this:StartMoving()
                this.isMoving = true
            end
        end)
        
        moveFrame:SetScript("OnMouseUp", function()
            local p = this.playerModel
            if arg1 == 'LeftButton' then
                this.isRotating = false
                this.rotation = p.rotation * 50
            elseif arg1 == 'RightButton' then
                this.isMoving = false
                this.pan_x = p.pos_x * 100
                this.pan_y = p.pos_y * 100
            else
                return
            end
            this:StopMovingOrSizing()
            this:SetAllPoints()
        end)
        
        moveFrame:SetScript("OnUpdate", function()
            local p = this.playerModel
            if this.isRotating then
                p.rotation = (this:GetCenter() + this.rotation
                              - p:GetCenter()) * 0.02
                p:SetRotation(p.rotation)
            end
            if this.isMoving then
                p.pos_x = (this:GetCenter() + this.pan_x - p:GetCenter()) * 0.01
                p.pos_y = (this:GetTop() + this.pan_y - p:GetTop()) * 0.01
                p:SetPosition(p.zoom, p.pos_x, p.pos_y)
            end
        end)
        
        moveFrame:SetScript("OnMouseWheel", function()
            local p = this.playerModel
            if arg1 > 0 then
                p.zoom = p.zoom + 0.5
            else
                p.zoom = p.zoom - 0.5
            end
            p:SetPosition(p.zoom, p.pos_x, p.pos_y)
        end)
        
        moveFrame:SetScript("OnShow", function()
            this:SetAllPoints()
        end)
        
        self.movement = moveFrame
    end
    
    moveFrame:EnableMouse(true)
    moveFrame:EnableMouseWheel(true)
    moveFrame:SetMovable(true)
    
    playerModel.rotation = 0.61 -- FROM: InspectModelFrame_OnLoad
    playerModel.pos_x = 0
    playerModel.pos_y = 0
    playerModel.zoom = 0
    
    moveFrame.pan_x = 0
    moveFrame.pan_y = 0
    moveFrame.rotation = playerModel.rotation * 50 -- to get the right rotation
    
    playerModel:SetRotation(playerModel.rotation)
    playerModel:RefreshUnit()
end

function Spyglass:LatchOn()
    if self.latched then return true end
    
    -- Quit if the inspect window doesn't exist
    if not _G.InspectFrame or not self:IsActive() then return false end
    
    if not self.inspectFrame and self:IsEventRegistered('ADDON_LOADED') then
        -- First time, so we won't need to catch the addon loading again
        self:UnregisterEvent('ADDON_LOADED')
    end
    
    -- Local references to the important frames
    self.inspectFrame = _G.InspectFrame
    tinsert(UISpecialFrames,self.inspectFrame:GetName()); 
    
    -- Attach player rank
    if not self.infoText then
        self.infoText = _G.InspectPaperDollFrame:CreateFontString('InspectInfoText',
                                                                  'BACKGROUND',
                                                                  'GameFontNormalSmall')
        self.infoText:SetPoint('TOP', 'InspectLevelText', 'BOTTOM', 0, -1)
    else
        self.infoText:Show()
    end
    
    self:GlueMovementFrame()
    self:GlueSummaryFrame()
    
    self:LatchButtons()
    
    -- Unregister certain events the original used, but we don't want it to use
    self.inspectFrame:UnregisterEvent('PLAYER_TARGET_CHANGED')
    self.inspectFrame:UnregisterEvent('PARTY_MEMBERS_CHANGED')
    
    -- Shifting the event handling over to us
    self:RegisterEvent('PLAYER_TARGET_CHANGED', 'TargetChanged')
    self:RegisterEvent('PARTY_MEMBERS_CHANGED', 'TargetChanged')
    
    self:RegisterEvent('UNIT_INVENTORY_CHANGED', 'UnitChangedInventory')
    self:RegisterEvent('UNIT_NAME_UPDATE', 'UnitChangedName')
    
    -- Hooks for adding extra information to the frames
    self:SecureHook('InspectFrame_UnitChanged', 'UnitChanged')
    self:SecureHook('InspectFrame_OnShow', 'UnitChanged')
    self:SecureHook('InspectFrame_Show', 'SetInspectUnit')
    
    -- Prevent closing the InspectFrame when we go out of inspect range
    --   Sloppy, but HookSecure can't overwrite, and we need to.
    if not self.oldOnUpdate then
        self.oldOnUpdate = _G.InspectFrame_OnUpdate
        _G.InspectFrame_OnUpdate = function() end
    end
    
    _G.InspectModelRotateLeftButton:Hide()
    _G.InspectModelRotateRightButton:Hide()
    
    --self:Print('Latched on.')
    return true
end

function Spyglass:LetGo()
    if not self.latched then return false end
    
    self.summaryFrame:Hide()
    self.infoText:Hide()
    
    self:ReleaseButtons()
    
    if self.oldOnUpdate then
        _G.InspectFrame_OnUpdate = self.oldOnUpdate
        self.oldOnUpdate = nil
    end
    
    -- Restoring event registration
    self.inspectFrame:RegisterEvent('PLAYER_TARGET_CHANGED')
    self.inspectFrame:RegisterEvent('PARTY_MEMBERS_CHANGED')
    
    return true
end

function Spyglass:SetSavedEquipment(enable)
    local prof = self.db.profile
    if enable then
        prof.storedEquipment = self.unitCache
    else
        self.unitCache = prof.storedEquipment
        prof.storedEquipment = {}
    end
    prof.storeEquipment = enable
end

local function LoadInspectInfo(unit, info)
    -- try to re-use old info tables
    if not info then
        -- These should never change, so we only need to get them once
        local class, localClass = UnitClass(unit)
        info = {
            name = UnitName(unit),
            race = UnitRace(unit),
            class = class,
            localClass = localClass,
        }
    end
    
    info.level = UnitLevel(unit)
    
    local association, title = GetGuildInfo(unit)
    info.association = association
    info.title = title
    
    info.equipment = IBL:GetUnitEquipment(unit)
    
    return info
end

function Spyglass:GetUnitInfo(unitName, update)
    local info = self.unitCache[unitName]
    if not update then return info end
    
    info = LoadInspectInfo(self.inspectFrame.unit, info)
    self.unitCache[unitName] = info
    return info
end

function Spyglass:RemoveUnitInfo(unitName)
    if self.unitCache[unitName] then
        self:Print(format(L['Removing player info: %s'], unitName))
        
        if self.unitName == unitName and self.inspectFrame:IsShown() then
            self:SetInspectUnit('player')
        end
        
        self.unitCache[unitName] = nil
    end
end

function Spyglass:ClearUnitInfo()
    if self.inspectFrame and self.inspectFrame:IsShown() then
        _G.HideUIPanel(self.inspectFrame)
    end
    
    self:Print(L['Clearing all user data'])
    
    local cache = self.unitCache
    for k, v in pairs(cache) do
        cache[k] = nil
    end
    
    self.db.profile.lastInspected = nil
    
    _G.collectgarbage('collect') -- clean up
end

function Spyglass:GetCachedPlayerNames()
    -- Create and sort a list of unit names
    local sortedNames = {}
    for name in pairs(self.unitCache) do
        sortedNames[#sortedNames+1] = name
    end
    table.sort(sortedNames)
    return sortedNames
end

local function CheckCache(unitName, backup)
    -- returns: index, if the given names were in cache
    if Spyglass.unitCache[unitName] then
        return unitName, true
    elseif Spyglass.unitCache[backup] then
        return backup, true
    end
    
    return UnitName('player'), false
end


function Spyglass:FinalizeSetInspect(unit, unitName)
    if not self.latched then
        if not self.inspectFrame then
            InspectFrame_LoadUI()
        end
        
        if not self.inspectFrame then
            -- This "shouldn't" be the case, but it just might be
            self.latched = self:LatchOn()
        end
    end
    
    --self:Print('Setting unit to:', unit,', with name:', unitName)
    
    -- If we switched to a target that we can't inspect, including targeting
    --   nobody at all, make InspectUI think we're inspecting outself.
    --   'player' was chosen because it's guaranteed to work with
    --   InspectFrame 100% of the time. We also mark an internal flag that
    --   we cannot inspect, telling us to use cached information.
    
    if not self.inInspectRange then
        local name, cached = CheckCache(unitName, self.unitName)
        if not cached then self.inInspectRange = true end
        self.inspectFrame.unit = 'player'
        self.unitName = name
    else
        self.inspectFrame.unit = unit
        self.unitName = unitName
    end
    
    self.currentInfo = self:GetUnitInfo(self.unitName, self.inInspectRange)
    --self:Print('REALLY Setting unit to:', self.inspectFrame.unit,', with name:', self.unitName)
    
    if not self.inspectFrame:IsShown() then
        _G.ShowUIPanel(self.inspectFrame)
    else
        -- A little messy, but we need to make InspectUI think it was called
        --   from itself.
        --this = self.inspectFrame
        -- The above breaks if we were called from the drop-down menus
        local save_unit = this.unit
        this.unit = self.inspectFrame.unit
        securecall('InspectFrame_UnitChanged')
        this.unit = save_unit
    end
end

function Spyglass:SetInspectPlayer(unitName)
    if not self:IsActive() then return end
    
    self.inInspectRange =
        (UnitName('target') == unitName and CanInspect('target'))
    
    self:FinalizeSetInspect('target', unitName)
end

function Spyglass:SetInspectUnit(unit)
    if not self:IsActive() then return end
    
    unit = unit or 'target'
    self.inInspectRange = CanInspect(unit)
    
    self:FinalizeSetInspect(unit, UnitName(unit))
end

function Spyglass:ShowWindow()
    if not self:IsActive() then return end
    
    -- Show the inspection window if it's not shown already; otherwise, hide it
    if self.inspectFrame and self.inspectFrame:IsShown() then
        _G.HideUIPanel(self.inspectFrame)
    else
        self:SetInspectUnit('target')
    end
end

-- Player's target changed
function Spyglass:TargetChanged()
    if not self:IsActive() or not self.inspectFrame:IsShown() then return end
    
    self:SetInspectUnit('target')
end

function Spyglass:UnitChangedInventory(unit)
    if not self:IsActive() or not self.inspectFrame:IsShown() then return end
    
    if unit == 'target' and self.inInspectRange then
        -- force an update of their inventory
        self.currentInfo = self:GetUnitInfo(self.unitName, CanInspect(unit))
        self:UnitChanged()
    end
end

function Spyglass:UnitChangedName(unit)
    if not self:IsActive() or not self.inspectFrame:IsShown() then return end
    
    if unit == 'target' and self.inInspectRange then
        -- reload info ... or something
        self.currentInfo = self:GetUnitInfo(self.unitName, CanInspect(unit))
        self:UnitChanged()
    end
end

-- Unit changed, so set per-unit information
function Spyglass:UnitChanged()
    if not self.inspectFrame:IsShown() then return end
    
    if not self.currentInfo then
        -- This is the case if we're initializing from the default inspect
        self.currentInfo = self:GetUnitInfo(self.unitName, self.inInspectRange)
    end
    
    local info = self.currentInfo
    
    -- Update Text
    _G.InspectNameText:SetText(self.unitName)
    _G.InspectLevelText:SetFormattedText(PLAYER_LEVEL, info.level, info.race,
                                         info.class)
    
    if info.association and info.title then
        self.infoText:SetFormattedText(GUILD_TITLE_TEMPLATE, info.title,
                                       '<'..info.association..'>')
    else
        self.infoText:SetText('')
    end
    
    -- Update Model
    if not self.inInspectRange then
        SetPortraitToTexture(_G.InspectFramePortrait, [[Interface\Icons\INV_Misc_Spyglass_03]])
        _G.InspectModelFrame:Hide()
    else
        SetPortraitTexture(_G.InspectFramePortrait, 'target')
        _G.InspectModelFrame:Show()
    end
    
    -- Update Items
    self:UpdateButtons(info)
    
    -- Show Bonuses
    if self.db.profile.showBonuses then
        self:Summarize(info)
    end
    
    self.db.profile.lastInspected = self.unitName
end
